@@ -8,22 +8,16 @@ import sys
 import traceback
 import uuid
 from functools import partial
-from typing import TYPE_CHECKING, Optional
+from typing import Optional, ClassVar, Awaitable
 
 import requests
 from tornado import ioloop
 
-from .contexts.default.context import DefaultContext
-from .lib.context import BeakerContext, autodiscover_contexts
-from .lib.jupyter_kernel_proxy import (KERNEL_SOCKETS, KERNEL_SOCKETS_NAMES,
-                                       InterceptionFilter, JupyterMessage,
-                                       KernelProxyManager)
-from .lib.utils import (message_handler, LogMessageEncoder, magic,
-                        handle_message, get_socket, execution_context, parent_message_context, get_parent_message)
-
-if TYPE_CHECKING:
-    from .lib.agent import BeakerAgent
-    from .lib.subkernels.base import BeakerSubkernel
+from beaker_kernel.lib.config import reset_config, config
+from beaker_kernel.lib.context import BeakerContext, autodiscover_contexts
+from beaker_kernel.lib.jupyter_kernel_proxy import InterceptionFilter, JupyterMessage, KernelProxyManager
+from beaker_kernel.lib.utils import (message_handler, LogMessageEncoder, magic,
+                        handle_message, get_socket, execution_context, parent_message_context)
 
 USER_RESPONSE_WAIT_TIME = 100
 
@@ -41,16 +35,17 @@ AVAILABLE_CONTEXTS = autodiscover_contexts()
 
 
 class BeakerKernel(KernelProxyManager):
-    implementation = "beaker-kernel"
-    implementation_version = "0.1"
-    banner = "Beaker Kernel"
+    implementation: ClassVar[str] = "beaker-kernel"
+    implementation_version: ClassVar[str] = "0.1"
+    banner: ClassVar[str] = "Beaker Kernel"
 
-    language_info = {
+    language_info: ClassVar[dict[str, str]] = {
         "mimetype": "text/plain",
         "name": "text",
         "file_extension": ".txt",
     }
 
+    jupyter_server: Optional[str]
     kernel_id: Optional[str]
     connection_file: Optional[str]
     context: Optional[BeakerContext]
@@ -59,8 +54,11 @@ class BeakerKernel(KernelProxyManager):
     user_responses: dict[str, str]
     debug_enabled: bool
     magic_commands: dict[str, callable]
+    ready: asyncio.Future
+    running_actions: dict[str, Awaitable]
 
     def __init__(self, session_config, kernel_id=None, connection_file=None):
+        self.jupyter_server = session_config.get("server", config.jupyter_server)
         self.kernel_id = kernel_id
         self.connection_file = connection_file
         self.debug_enabled = False
@@ -68,6 +66,7 @@ class BeakerKernel(KernelProxyManager):
         self.magic_commands = {}
         self.internal_executions = set()
         self.subkernel_execution_tracking = {}
+        self.running_actions = {}
         super().__init__(session_config, session_id=f"{kernel_id}_session")
         self.register_magic_commands()
         self.add_base_intercepts()
@@ -75,7 +74,26 @@ class BeakerKernel(KernelProxyManager):
         self.user_responses = dict()
         # Initialize context (Using the event loop to simulate `await`ing the async func in non-async setup)
         event_loop = asyncio.get_event_loop()
-        event_loop.run_until_complete(self.set_context("default", {}))
+        context_task = event_loop.create_task(self.start_default_context())
+        context_task.add_done_callback(lambda task: None)
+
+    async def start_default_context(self):
+        default_context = os.environ.get('BEAKER_DEFAULT_CONTEXT')
+        default_context_payload = os.environ.get('BEAKER_DEFAULT_CONTEXT_PAYLOAD', "{}")
+        if default_context_payload:
+            try:
+                default_context_payload = json.loads(default_context_payload)
+            except json.JSONDecodeError:
+                default_context_payload = {}
+        if not default_context:
+            sorted_contexts = sorted(autodiscover_contexts().items(), key=lambda item: item[1].WEIGHT)
+            first_context = sorted_contexts[0]
+            default_context, context_cls = first_context
+            default_context_payload = context_cls.default_payload()
+        if not default_context:
+            default_context = "default"
+            default_context_payload = {}
+        await self.set_context(default_context, None)
 
     def add_base_intercepts(self):
         """
@@ -96,6 +114,9 @@ class BeakerKernel(KernelProxyManager):
         )
         self.server.intercept_message("shell", "execute_reply", self.post_execute)
         self.server.intercept_message("stdin", "input_reply", self.input_reply)
+        self.server.intercept_message("shell", "set_agent_model", self.set_agent_model)
+        self.server.intercept_message("shell", "reset_request", self.reset_kernel)
+        self.server.intercept_message("control", "interrupt_request", self.interrupt)
 
     def register_magic_commands(self):
         for _, method in inspect.getmembers(self, lambda member: inspect.ismethod(member) and hasattr(member, "_magic_prefix")):
@@ -140,7 +161,7 @@ class BeakerKernel(KernelProxyManager):
             content={},
             parent_header=parent_header,
         )
-        self.stdout(f"Context switch complete.", parent_header=parent_header)
+        self.stdout("Context switch complete.", parent_header=parent_header)
 
         # TODO: Return the new context info
         return None
@@ -228,7 +249,7 @@ class BeakerKernel(KernelProxyManager):
             run_info = {}
         run_info.update(kwargs)
         with open(self.connection_file, "w") as connection_file:
-                json.dump(run_info, connection_file, indent=2)
+            json.dump(run_info, connection_file, indent=2)
 
     async def set_context(self, context_name, context_info, language="python3", parent_header={}):
 
@@ -241,11 +262,17 @@ class BeakerKernel(KernelProxyManager):
         if self.context:
             self.context.cleanup()
 
-        config = {
+        if context_info is None:
+            default_payload = context_cls.default_payload()
+            if isinstance(default_payload, str):
+                default_payload = json.loads(default_payload)
+            context_info = default_payload
+
+        context_config = {
             "language": language,
             "context_info": context_info
         }
-        self.context = context_cls(beaker_kernel=self, config=config)
+        self.context = context_cls(beaker_kernel=self, config=context_config)
         await self.context.setup(context_info=context_info, parent_header=parent_header)
         subkernel = self.context.subkernel
         kernel_setup_func = getattr(subkernel, "setup", None)
@@ -322,6 +349,27 @@ class BeakerKernel(KernelProxyManager):
             data = message.parts
         return data
 
+    @message_handler
+    async def interrupt(self, _message):
+        try:
+            subkernel_id = self.context.subkernel.jupyter_id
+            print(f"Interrupting connected subkernel: {subkernel_id}")
+            requests.post(
+                f"{self.context.beaker_kernel.jupyter_server}/api/kernels/{subkernel_id}/interrupt",
+                headers={"Authorization": f"token {config.jupyter_token}"},
+            )
+        except requests.exceptions.HTTPError as err:
+            logger.error(f"Subkernel cannot be interrupted.\nDetails:\n  {err.request.body}", exc_info=err)
+            return None
+
+        for key, value in list(self.running_actions.items()):
+            if inspect.iscoroutine(value):
+                value.throw(asyncio.CancelledError, "Execution interrupted by user.")
+            elif isinstance(value, asyncio.Future):
+                value.cancel(msg="Execution interrupted by user.")
+            del self.running_actions[key]
+        return None
+
     async def prompt_user(self, query, parent_message=None):
         msg_id = str(uuid.uuid4())
         self.send_response(
@@ -388,6 +436,7 @@ class BeakerKernel(KernelProxyManager):
     @message_handler
     async def llm_request(self, message):
         # Send "code" to LLM Agent. The "code" is actually the LLM query
+        from archytas.exceptions import AuthenticationError
         content = message.content
         request = content.get("request", None)
         if not request:
@@ -395,55 +444,83 @@ class BeakerKernel(KernelProxyManager):
         if not self.context:
             raise Exception("Context has not been set")
         setattr(self.context, "current_llm_query", request)
+        request_key = f"llm_query:{message.header['msg_id']}"
         try:
-            # Before starting ReAct loop, replace thought handler with partial func with parent_header so we can track thoughts
-            # with parent_message_context(parent_message=message):
-                self.context.agent.thought_handler = partial(self.handle_thoughts, parent_header=message.header)
+            try:
+                # Before starting ReAct loop, replace thought handler with partial func with parent_header so we can track
+                # thoughts
+                if self.context.agent:
+                    self.context.agent.thought_handler = partial(self.handle_thoughts, parent_header=message.header)
                 self.debug("llm_query", request, parent_header=message.header)
-                result = await self.context.agent.react_async(request, react_context={"message": message})
-        except Exception as err:
-            error_text = f"""LLM Error:
+                task = asyncio.create_task(self.context.agent.react_async(request, react_context={"message": message}))
+                self.running_actions[request_key] = task
+                result = await task
+            except AuthenticationError as err:
+                self.send_response(
+                    stream="iopub",
+                    msg_or_type="llm_auth_failure",
+                    content={
+                        "msg": str(err),
+                    },
+                    parent_header=message.header,
+                )
+                return
+            except asyncio.CancelledError as err:
+                self.send_response(
+                    stream="iopub",
+                    msg_or_type="stream",
+                    content={
+                        "name": "stderr",
+                        "text": "Request interrupted.",
+                    },
+                    parent_header=message.header,
+                )
+                raise
+            except Exception as err:
+                error_text = f"""LLM Error:
 {err}
 
 {traceback.format_exc()}
 """
-            stream_content = {"name": "stderr", "text": error_text}
-            self.send_response(
-                "iopub", "stream", stream_content, parent_header=message.header
-            )
-            raise
-        try:
-            # Normalize result
-            if isinstance(result, (str, bytes, bytearray)):
-                data = json.loads(result)
-            else:
-                data = result
-
-            if isinstance(data, dict) and data.get("action") == "code_cell":
-                stream_content = {
-                    "language": data.get("language"),
-                    "code": data.get("content"),
-                }
+                stream_content = {"name": "stderr", "text": error_text}
                 self.send_response(
-                    "iopub", "code_cell", stream_content, parent_header=message.header
+                    "iopub", "stream", stream_content, parent_header=message.header
                 )
-            else:
-                stream_content = {"name": "response_text", "text": f"{data}"}
+                raise
+            try:
+                # Normalize result
+                if isinstance(result, (str, bytes, bytearray)):
+                    data = json.loads(result)
+                else:
+                    data = result
+
+                if isinstance(data, dict) and data.get("action") == "code_cell":
+                    stream_content = {
+                        "language": data.get("language"),
+                        "code": data.get("content"),
+                    }
+                    self.send_response(
+                        "iopub", "code_cell", stream_content, parent_header=message.header
+                    )
+                else:
+                    stream_content = {"name": "response_text", "text": f"{data}"}
+                    self.send_response(
+                        "iopub", "llm_response", stream_content, parent_header=message.header
+                    )
+            except (
+                json.JSONDecodeError
+            ):  # If response is not a json, it's just text so treat it like text
+                stream_content = {"name": "response_text", "text": f"{result}"}
                 self.send_response(
                     "iopub", "llm_response", stream_content, parent_header=message.header
                 )
-        except (
-            json.JSONDecodeError
-        ):  # If response is not a json, it's just text so treat it like text
-            stream_content = {"name": "response_text", "text": f"{result}"}
-            self.send_response(
-                "iopub", "llm_response", stream_content, parent_header=message.header
-            )
+            finally:
+                # When done, put thought handler back to default to not potentially cause confused thoughts.
+                self.context.agent.thought_handler = self.handle_thoughts
+                setattr(self.context, "current_llm_query", None)
         finally:
-            # When done, put thought handler back to default to not potentially cause confused thoughts.
-            self.context.agent.thought_handler = self.handle_thoughts
-            setattr(self.context, "current_llm_query", None)
-
+            if request_key in self.running_actions:
+                del self.running_actions[request_key]
 
     @message_handler
     async def context_info_request(self, message):
@@ -509,6 +586,31 @@ class BeakerKernel(KernelProxyManager):
         parent_id = message.parent_header["msg_id"]
         self.user_responses[parent_id] = content["value"]
 
+    @message_handler
+    async def set_agent_model(self, message):
+        provider_id = message.content.get("provider_id", None)
+        model_config = message.content.get("model_config", None)
+        if provider_id or model_config:
+            model = config.get_model(provider_id=provider_id, model_config=model_config)
+        else:
+            # Refresh the cached config object so we pick up any changes to the config at the source
+            reset_config()
+            model = config.get_model()
+        if model:
+            self.context.agent.model = model
+
+    @message_handler
+    async def reset_kernel(self, message):
+        reset_config()
+        await self.set_context(
+            self.context.SLUG,
+            self.context.config,
+            language=self.context.subkernel.SLUG,
+            parent_header=message.header
+        )
+        return True
+
+
 # Provided for backwards compatibility
 LLMKernel = BeakerKernel
 
