@@ -142,11 +142,18 @@ class BeakerContext:
             subkernel.KERNEL_NAME: subkernel
             for subkernel in autodiscover("subkernels").values()
         }
-        url = urllib.parse.urljoin(config.JUPYTER_SERVER, "/api/kernels")
+        subkernel_opts = {
+            subkernel.SLUG: subkernel
+            for subkernel in autodiscover("subkernels").values()
+        }
+        if language not in kernel_opts and language in subkernel_opts:
+            language = subkernel_opts[language].KERNEL_NAME
+
+        url = urllib.parse.urljoin(self.beaker_kernel.jupyter_server, "/api/kernels")
         res = requests.post(
             url,
             json={"name": language, "path": ""},
-            headers={"Authorization": f"token {config.JUPYTER_TOKEN}"},
+            headers={"Authorization": f"token {config.jupyter_token}"},
         )
         kernel_info = res.json()
         self.beaker_kernel.update_running_kernels()
@@ -292,7 +299,7 @@ class BeakerContext:
         the context.
         E.g. For "beaker_kernel.contexts.pandas" the slug would be "pandas"
         """
-        if self.SLUG:
+        if hasattr(self, "SLUG"):
             return self.SLUG
 
         package_str = inspect.getmodule(self).__package__
