@@ -13,11 +13,11 @@ from functools import wraps, update_wrapper
 from pathlib import Path
 from typing import Any, TYPE_CHECKING, Callable, List
 
-from archytas.tool_utils import tool
+from archytas.models.base import BaseArchytasModel
+from archytas.exceptions import AuthenticationError
 
 from .jupyter_kernel_proxy import ( KERNEL_SOCKETS, KERNEL_SOCKETS_NAMES,
                                    JupyterMessage, JupyterMessageTuple)
-from .config import config
 
 
 logger = logging.getLogger(__name__)
@@ -45,6 +45,16 @@ def find_file_along_path(filename: str, start_path: Path | str | None = None) ->
             return potential_file
     return None
 
+class DefaultModel(BaseArchytasModel):
+    def initialize_model(self, **kwargs):
+        return
+
+    def invoke(self, input, *, config=None, stop=None, **kwargs):
+        raise AuthenticationError("Model not found or misconfigured. Please check your provider configuration.")
+
+    def ainvoke(self, input, *, config=None, stop=None, **kwargs):
+        raise AuthenticationError("Model not found or misconfigured. Please check your provider configuration.")
+
 class ExecutionError(RuntimeError):
     def __init__(self, ename: str, evalue: str, traceback: List[str]) -> None:
         super().__init__(ename, evalue, traceback)
@@ -86,24 +96,29 @@ class handle_message(AbstractAsyncContextManager):
 
     async def __aexit__(self, exc_type, exc_value, traceback_value):
         """Raise any exception triggered within the runtime context."""
-        if exc_type:
-            formatted_tb = traceback.format_exception(exc_type, exc_value, traceback_value)
-            sys.stderr.write("\n".join(formatted_tb))
-            sys.stderr.flush()
-            reply_content = {
-                "status": "error",
-                "execution_count": None,
-                "ename": exc_type.__name__,
-                "evalue": str(exc_value),
-                "traceback": formatted_tb,
-            }
-
-        else:
-            reply_content = {
-                "status": "ok",
-                "execution_count": None,
-                "return": self.return_val,
-            }
+        match exc_type:
+            case asyncio.CancelledError():
+                reply_content = {
+                    "status": "aborted",
+                    "execution_count": None,
+                }
+            case BaseException():
+                formatted_tb = traceback.format_exception(exc_type, exc_value, traceback_value)
+                sys.stderr.write("\n".join(formatted_tb))
+                sys.stderr.flush()
+                reply_content = {
+                    "status": "error",
+                    "execution_count": None,
+                    "ename": exc_type.__name__,
+                    "evalue": str(exc_value),
+                    "traceback": formatted_tb,
+                }
+            case _:
+                reply_content = {
+                    "status": "ok",
+                    "execution_count": None,
+                    "return": self.return_val,
+                }
 
         self.beaker_kernel.send_response(
             "shell", self.reply_type, reply_content, parent_header=self.message.header, parent_identities=self.message.identities
@@ -164,7 +179,7 @@ def action(action_name: str|None=None, docs: str|None=None, default_payload=None
     """
     Method decorator to identify and register context actions.
     """
-    if enabled is not None and enabled():
+    if enabled is not None and not enabled():
         def disable(_fn):
             def disabled_message(*args, **kwargs):
                 raise RuntimeError("This action is disabled.")
